"""
User helper
"""

# Django
from django.contrib.auth.models import User

# AA SRP
from aasrp.models import UserSetting


def get_user_settings(user: User) -> UserSetting:
    """
    Get a user's settings or create them

    :param user:
    :type user:
    :return:
    :rtype:
    """

    return UserSetting.objects.get_or_create(user=user)[0]



# Generated by Django 4.2.8 on 2023-12-11 22:11

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("aasrp", "0012_alter_fleettype_options_alter_insurance_options_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="setting",
            name="loss_value_source",
            field=models.CharField(
                choices=[
                    ("fittedValue", "Fitted value (Ship and Fitting)"),
                    ("totalValue", "Total value (Ship, Fitting and Cargo)"),
                ],
                default="totalValue",
                help_text="The source for the loss value of a killmail. Fitted value is the value of the ship and its fitting. Total value is the value of the ship, its fitting and the cargo. (Default: Total value)",
                max_length=11,
                verbose_name="Loss value source",
            ),
        ),
    ]



"""
AA-SRP Ajax URLs
"""

# Django
from django.urls import include, path

# AA SRP
from aasrp.views import ajax

urls = [
    path(
        route="ajax/",
        view=include(
            [
                path(
                    # Get active SRP links
                    route="active-srp-links-data/",
                    view=ajax.dashboard_srp_links_data,
                    name="ajax_dashboard_srp_links_data",
                ),
                path(
                    # Get all SRP links
                    route="active-srp-links-data/all/",
                    view=ajax.dashboard_srp_links_data,
                    kwargs={"show_all_links": True},
                    name="ajax_dashboard_srp_links_all_data",
                ),
                path(
                    # Get all SRP request for the current user
                    route="user-srp-requests-data/",
                    view=ajax.dashboard_user_srp_requests_data,
                    name="ajax_dashboard_user_srp_requests_data",
                ),
                path(
                    # Get all SRP requests for the current SRP link
                    route="srp-link/<str:srp_code>/view-srp-requests-data/",
                    view=ajax.srp_link_view_requests_data,
                    name="ajax_srp_link_view_requests_data",
                ),
                path(
                    # Get addition information for the current SRP request
                    route=(
                        "srp-link/<str:srp_code>/srp-request/"
                        "<str:srp_request_code>/view-additional-information-data/"
                    ),
                    view=ajax.srp_request_additional_information,
                    name="ajax_srp_request_additional_information",
                ),
                path(
                    # Change the SRP payout amount
                    route=(
                        "srp-link/<str:srp_code>/srp-request/"
                        "<str:srp_request_code>/change-srp-payout/"
                    ),
                    view=ajax.srp_request_change_payout,
                    name="ajax_srp_request_change_payout",
                ),
                path(
                    # Approve the SRP request
                    route=(
                        "srp-link/<str:srp_code>/srp-request/"
                        "<str:srp_request_code>/approve/"
                    ),
                    view=ajax.srp_request_approve,
                    name="ajax_srp_request_approve",
                ),
                path(
                    # Bulk approve SRP requests
                    route=("srp-link/<str:srp_code>/srp-request/approve/bulk/"),
                    view=ajax.srp_requests_bulk_approve,
                    name="ajax_srp_requests_bulk_approve",
                ),
                path(
                    # Deny the SRP request
                    route=(
                        "srp-link/<str:srp_code>/srp-request/"
                        "<str:srp_request_code>/deny/"
                    ),
                    view=ajax.srp_request_deny,
                    name="ajax_srp_request_deny",
                ),
                path(
                    # Remove the SRP request
                    route=(
                        "srp-link/<str:srp_code>/srp-request/"
                        "<str:srp_request_code>/remove/"
                    ),
                    view=ajax.srp_request_remove,
                    name="ajax_srp_request_remove",
                ),
                path(
                    # Bulk remove SRP requests
                    route=("srp-link/<str:srp_code>/srp-request/remove/bulk/"),
                    view=ajax.srp_requests_bulk_remove,
                    name="ajax_srp_requests_bulk_remove",
                ),
            ]
        ),
    ),
]



# Django
from django.db import migrations

# Alliance Auth (External Libs)
from app_utils.app_settings import clean_setting

srp_team_discord_channel_id = clean_setting(
    name="AASRP_SRP_TEAM_DISCORD_CHANNEL", default_value=None, required_type=int
)


def on_migrate(apps, schema_editor):
    """
    Create default settings on migration
    :param apps:
    :param schema_editor:
    :return:
    """

    Setting = apps.get_model("aasrp", "Setting")
    db_alias = schema_editor.connection.alias

    Setting.objects.using(db_alias).create(
        pk=1,
        srp_team_discord_channel_id=srp_team_discord_channel_id,
    )


def on_migrate_zero(apps, schema_editor):
    """
    Remove default settings on migration to zero
    :param apps:
    :param schema_editor:
    :return:
    """

    Setting = apps.get_model("aasrp", "Setting")
    db_alias = schema_editor.connection.alias
    Setting.objects.using(db_alias).delete()


class Migration(migrations.Migration):
    """
    Run migrations
    """

    dependencies = [
        ("aasrp", "0010_model_changes"),
    ]

    operations = [migrations.RunPython(on_migrate, on_migrate_zero)]



"""
AA-SRP URL Configuration
"""

# Django
from django.urls import include, path

# AA SRP
from aasrp import __app_name__
from aasrp.constants import INTERNAL_URL_PREFIX
from aasrp.urls import ajax, views

app_name: str = __app_name__

urlpatterns = [
    # Views
    path(route="", view=include(views.urls)),
    # Ajax calls
    path(route=f"{INTERNAL_URL_PREFIX}/", view=include(ajax.urls)),
]



"""
Tests for aasrp.helper.character
"""

# Django
from django.contrib.auth.models import Group
from django.test import TestCase

# Alliance Auth
from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter

# Alliance Auth (External Libs)
from app_utils.testing import (
    add_character_to_user,
    create_eve_character,
    create_fake_user,
)

# AA SRP
from aasrp.helper.character import (
    get_formatted_character_name,
    get_main_for_character,
    get_user_for_character,
)
from aasrp.helper.eve_images import get_character_portrait_from_evecharacter
from aasrp.helper.icons import copy_to_clipboard_icon
from aasrp.models import get_sentinel_user


class TestSentinelUser(TestCase):
    """
    Test the sentinel user
    """

    def test_sentinel_user(self):
        """
        Test that we get 'deleted' as username for the sentinel user

        :return:
        :rtype:
        """

        sentinel_user = get_sentinel_user()

        self.assertEqual(first=sentinel_user.username, second="deleted")


class TestGetFormattedCharacterName(TestCase):
    """
    Tests for get_formatted_character_name
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()
        cls.group = Group.objects.create(name="Enterprise Crew")

        cls.user_main_character = create_fake_user(
            character_id=1001, character_name="William T. Riker"
        )

        cls.alt_character = create_eve_character(
            character_id=1002, character_name="Thomas Riker"
        )

        add_character_to_user(user=cls.user_main_character, character=cls.alt_character)

        cls.character_without_profile = create_eve_character(
            character_id=1003, character_name="Christopher Pike"
        )

    def test_should_return_formatted_character_name(self):
        """
        Test should return a formatted character name

        :return:
        :rtype:
        """

        html = get_formatted_character_name(character=self.alt_character)
        expected_html = (
            "<small class='text-muted'>"
            f"{self.alt_character.alliance_ticker} "
            f"[{self.alt_character.corporation_ticker}] "
            f"</small><br>{self.alt_character.character_name}"
        )

        self.assertEqual(first=html, second=expected_html)

    def test_should_return_formatted_character_name_with_copy_icon(self):
        """
        Test should return a formatted character name with copy icon

        :return:
        :rtype:
        """

        self.maxDiff = None

        html = get_formatted_character_name(
            character=self.alt_character, with_copy_icon=True
        )

        icon = copy_to_clipboard_icon(
            data=self.alt_character.character_name,
            title="Copy character name to clipboard",
        )

        copy_icon = f"<sup>{icon}</sup>"

        expected_html = (
            "<small class='text-muted'>"
            f"{self.alt_character.alliance_ticker} "
            f"[{self.alt_character.corporation_ticker}] "
            f"</small><br>{self.alt_character.character_name}{copy_icon}"
        )

        self.assertEqual(first=html, second=expected_html)

    def test_should_return_formatted_character_name_with_portrait(self):
        """
        Test should return a formatted character name with a portrait

        :return:
        :rtype:
        """

        html = get_formatted_character_name(
            character=self.alt_character, with_portrait=True, inline=False
        )

        formatted_character_name = get_formatted_character_name(
            character=self.alt_character
        )

        character_portrait_html = get_character_portrait_from_evecharacter(
            character=self.alt_character, size=32, as_html=True
        )

        expected_html = (
            f"{character_portrait_html}<br>"
            "<span class='aasrp-character-portrait-character-name d-inline-block align-middle'>"
            f"{formatted_character_name}"
            "</span>"
        )

        self.assertEqual(first=html, second=expected_html)

    def test_should_return_formatted_character_name_with_portrait_inline(self):
        """
        Test should return a formatted character name with a portrait (inline)

        :return:
        :rtype:
        """

        html = get_formatted_character_name(
            character=self.alt_character, with_portrait=True
        )

        formatted_character_name = get_formatted_character_name(
            character=self.alt_character
        )

        character_portrait_html = get_character_portrait_from_evecharacter(
            character=self.alt_character, size=32, as_html=True
        )

        expected_html = (
            f"{character_portrait_html}"
            "<span class='aasrp-character-portrait-character-name d-inline-block align-middle'>"
            f"{formatted_character_name}"
            "</span>"
        )

        self.assertEqual(first=html, second=expected_html)

    def test_should_return_unknown_character_name(self):
        """
        Test should return "Unknown Character" for a broken EveCharacter object,
        e.g., when the EveCharacter object is None

        :return:
        :rtype:
        """

        html = get_formatted_character_name(character=None)

        expected_html = (
            "<span class='aasrp-character-portrait-character-name d-inline-block align-middle'>"
            "Unknown character"
            "</span>"
        )

        self.assertEqual(first=html, second=expected_html)


class TestGetMainForCharacter(TestCase):
    """
    Testing for get_main_for_character
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()
        cls.group = Group.objects.create(name="Enterprise Crew")

        cls.user_main_character = create_fake_user(
            character_id=1001, character_name="William T. Riker"
        )

        cls.alt_character = create_eve_character(
            character_id=1002, character_name="Thomas Riker"
        )

        cls.alt_character_2 = create_eve_character(
            character_id=1004, character_name="Jean Luc Riker"
        )

        add_character_to_user(user=cls.user_main_character, character=cls.alt_character)
        add_character_to_user(
            user=cls.user_main_character, character=cls.alt_character_2
        )

        cls.character_without_profile = create_eve_character(
            character_id=1003, character_name="Christopher Pike"
        )

    def test_get_main_for_character_returns_none(self):
        """
        Test if we get `None` as the main character

        :return:
        :rtype:
        """

        main_character = get_main_for_character(
            character=self.character_without_profile
        )

        self.assertIsNone(obj=main_character)

    def test_get_main_for_character_raises_exception_related_object_does_not_exist(
        self,
    ):
        """
        Test if we get `EveCharacter.userprofile.RelatedObjectDoesNotExist` as exception

        :return:
        :rtype:
        """

        get_main_for_character(character=self.character_without_profile)

        self.assertRaises(
            expected_exception=EveCharacter.userprofile.RelatedObjectDoesNotExist
        )

    def test_get_main_for_character_raises_exception_related_object_does_not_exist_2(
        self,
    ):
        """
        Test if we get `CharacterOwnership.user.RelatedObjectDoesNotExist` as exception

        :return:
        :rtype:
        """

        self.alt_character_2.character_ownership.user = None

        get_main_for_character(character=self.alt_character_2)

        self.assertRaises(
            expected_exception=CharacterOwnership.user.RelatedObjectDoesNotExist
        )

    def test_get_main_for_character_returns_main_character(self):
        """
        Test if we get the main character

        :return:
        :rtype:
        """

        main_character = get_main_for_character(character=self.alt_character)

        self.assertEqual(
            first=main_character.character_name,
            second=self.user_main_character.profile.main_character.character_name,
        )


class TestGetUserForCharacter(TestCase):
    """
    Tests for get_user_for_character
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()
        cls.group = Group.objects.create(name="Enterprise Crew")

        cls.user_main_character = create_fake_user(
            character_id=1001, character_name="William T. Riker"
        )

        cls.alt_character = create_eve_character(
            character_id=1002, character_name="Thomas Riker"
        )

        cls.alt_character_2 = create_eve_character(
            character_id=1004, character_name="Jean Luc Riker"
        )

        add_character_to_user(user=cls.user_main_character, character=cls.alt_character)
        add_character_to_user(
            user=cls.user_main_character, character=cls.alt_character_2
        )

        cls.character_without_profile = create_eve_character(
            character_id=1003, character_name="Christopher Pike"
        )

    def test_get_user_for_character_returns_sentinel_user(self):
        """
        Test if we get the sentinel user

        :return:
        :rtype:
        """

        returned_user = get_user_for_character(character=self.character_without_profile)
        sentinel_user = get_sentinel_user()

        self.assertEqual(first=returned_user, second=sentinel_user)

    def test_get_user_for_character_raises_exception_related_object_does_not_exist(
        self,
    ):
        """
        Test if we get `EveCharacter.userprofile.RelatedObjectDoesNotExist` as exception

        :return:
        :rtype:
        """

        get_user_for_character(character=self.character_without_profile)

        self.assertRaises(
            expected_exception=EveCharacter.userprofile.RelatedObjectDoesNotExist
        )

    def test_get_user_for_character_returns_user(self):
        """
        Test if we get a `User` object

        :return:
        :rtype:
        """

        returned_user = get_user_for_character(character=self.alt_character)

        self.assertEqual(
            first=returned_user, second=self.user_main_character.profile.user
        )

    def test_get_user_for_character_returns_sentinel_user_for_none(self):
        """
        Test if we get the sentinel user when user profile is None

        :return:
        :rtype:
        """

        self.alt_character_2.character_ownership.user = None

        returned_user = get_user_for_character(character=self.alt_character_2)
        expected_user = get_sentinel_user()

        self.assertEqual(first=returned_user, second=expected_user)
        self.assertRaises(
            expected_exception=CharacterOwnership.user.RelatedObjectDoesNotExist
        )



"""
Some helper functions, so we don't mess up other files too much
"""

# Django
from django.core.handlers.wsgi import WSGIRequest
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.functional import Promise
from django.utils.safestring import SafeString
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.authentication.decorators import permissions_required

# AA SRP
from aasrp.models import SrpLink, SrpRequest


def _create_button(  # pylint: disable=too-many-arguments, too-many-positional-arguments
    url: str,
    btn_class: str,
    icon_class: str,
    title: str,
    modal_id: str = None,
    data_name: str = None,
):
    """
    Create a button with the given parameters

    :param url:
    :type url:
    :param btn_class:
    :type btn_class:
    :param icon_class:
    :type icon_class:
    :param title:
    :type title:
    :param modal_id:
    :type modal_id:
    :param data_name:
    :type data_name:
    :return:
    :rtype:
    """

    href_or_modal_attrs = (
        f'data-bs-toggle="modal" data-bs-target="#{modal_id}" data-name="{data_name}" data-url="{url}"'
        if modal_id
        else f'href="{url}"'
    )

    return (
        f'<a class="btn {btn_class} btn-sm btn-icon-aasrp" '
        f'title="{title}" data-bs-tooltip="aa-srp" {href_or_modal_attrs}>'
        f'<i class="{icon_class}"></i></a>'
    )


def dashboard_action_icons(request: WSGIRequest, srp_link: SrpLink) -> str:
    """
    Getting the action buttons for the dashboard view

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :return:
    :rtype:
    """

    actions = ""

    # Active SRP link
    if srp_link.srp_status == SrpLink.Status.ACTIVE:
        # Add SRP request button
        actions += _create_button(
            url=reverse(viewname="aasrp:request_srp", args=[srp_link.srp_code]),
            btn_class="btn btn-success",
            icon_class="fa-solid fa-hand-holding-dollar",
            title=_("Request SRP"),
        )

    # Check if the user has permission to manage SRP links or requests
    if request.user.has_perm(perm="aasrp.manage_srp") or request.user.has_perm(
        perm="aasrp.manage_srp_requests"
    ):
        # Add view SRP requests button
        actions += (
            _create_button(
                url=reverse(
                    viewname="aasrp:view_srp_requests", args=[srp_link.srp_code]
                ),
                btn_class="btn btn-primary",
                icon_class="fa-solid fa-eye",
                title=_("View SRP requests"),
            )
            + "<br>"
        )

        # Whether the SRP link is active or closed, we can edit it
        if srp_link.srp_status != SrpLink.Status.COMPLETED and request.user.has_perm(
            "aasrp.manage_srp"
        ):
            # Check if the SRP status is active
            if srp_link.srp_status == SrpLink.Status.ACTIVE:
                # Add AAR link button
                actions += _create_button(
                    url=reverse(
                        viewname="aasrp:edit_srp_link", args=[srp_link.srp_code]
                    ),
                    btn_class="btn btn-info",
                    icon_class="fa-regular fa-newspaper",
                    title=_("Add/Change AAR link"),
                )
                # Add disable SRP link button
                actions += _create_button(
                    url=reverse(
                        viewname="aasrp:disable_srp_link", args=[srp_link.srp_code]
                    ),
                    btn_class="btn btn-warning",
                    icon_class="fa-solid fa-ban",
                    title=_("Disable SRP link"),
                    modal_id="disable-srp-link",
                    data_name=f"{srp_link.srp_name} ({srp_link.srp_code})",
                )

            # Check if the SRP status is closed
            if srp_link.srp_status == SrpLink.Status.CLOSED:
                # Add enable SRP link button
                actions += _create_button(
                    url=reverse(
                        viewname="aasrp:enable_srp_link", args=[srp_link.srp_code]
                    ),
                    btn_class="btn btn-success",
                    icon_class="fa-solid fa-check",
                    title=_("Enable SRP link"),
                    modal_id="enable-srp-link",
                    data_name=f"{srp_link.srp_name} ({srp_link.srp_code})",
                )

            # Add delete SRP link button
            actions += _create_button(
                url=reverse(viewname="aasrp:delete_srp_link", args=[srp_link.srp_code]),
                btn_class="btn btn-danger",
                icon_class="fa-regular fa-trash-can",
                title=_("Remove SRP link"),
                modal_id="delete-srp-link",
                data_name=f"{srp_link.srp_name} ({srp_link.srp_code})",
            )

    return actions


def get_srp_request_status_icon(
    request: WSGIRequest, srp_request: SrpRequest  # pylint: disable=unused-argument
) -> str:
    """
    Get status icon for srp request

    :param request:
    :type request:
    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    status_icons = {
        SrpRequest.Status.PENDING: ("btn-info", "fa-clock", _("SRP request pending")),
        SrpRequest.Status.APPROVED: (
            "btn-success",
            "fa-thumbs-up",
            _("SRP request approved"),
        ),
        SrpRequest.Status.REJECTED: (
            "btn-danger",
            "fa-thumbs-down",
            _("SRP request rejected"),
        ),
    }

    btn_class, icon_class, request_status_icon_title = status_icons.get(
        srp_request.request_status, ("btn-info", "fa-clock", _("SRP request pending"))
    )

    srp_request_status_icon = (
        f'<button class="btn {btn_class} btn-sm btn-icon-aasrp btn-icon-aasrp-status cursor-default" '
        f'title="{request_status_icon_title}" data-bs-tooltip="aa-srp">'
        f'<i class="fa-solid {icon_class}"></i>'
        "</button>"
    )

    return srp_request_status_icon


def get_srp_request_details_icon(
    request: WSGIRequest,  # pylint: disable=unused-argument
    srp_link: SrpLink,
    srp_request: SrpRequest,
) -> str:
    """
    Get details icon for an SRP request

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    button_request_details_url = reverse(
        viewname="aasrp:ajax_srp_request_additional_information",
        args=[srp_link.srp_code, srp_request.request_code],
    )

    title = _("SRP request details")

    srp_request_details_icon = (
        f'<button data-link="{button_request_details_url}" '
        'data-bs-tooltip="aa-srp" '
        'data-bs-toggle="modal" '
        'data-bs-target="#srp-request-details" '
        'class="btn btn-primary btn-sm btn-icon-aasrp" '
        f'title="{title}"><i class="fa-solid fa-circle-info"></i></button>'
    )

    return srp_request_details_icon


def get_srp_request_accept_icon(
    request: WSGIRequest,  # pylint: disable=unused-argument
    srp_link: SrpLink,
    srp_request: SrpRequest,
) -> str:
    """
    Get accept icon for an SRP request

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    button_request_accept_url = reverse(
        viewname="aasrp:ajax_srp_request_approve",
        args=[srp_link.srp_code, srp_request.request_code],
    )

    button_request_accept_state = (
        ' disabled="disabled"'
        if srp_request.request_status == SrpRequest.Status.APPROVED
        else ""
    )

    modal_target = (
        "#srp-request-accept-rejected"
        if srp_request.request_status == SrpRequest.Status.REJECTED
        else "#srp-request-accept"
    )

    icon = '<i class="fa-solid fa-check"></i>'
    title = _("Accept SRP request")

    srp_request_accept_icon = (
        f'<button data-link="{button_request_accept_url}" '
        'data-bs-tooltip="aa-srp" '
        'data-bs-toggle="modal" '
        f'data-bs-target="{modal_target}" '
        'class="btn btn-success btn-sm btn-icon-aasrp" '
        f'title="{title}"{button_request_accept_state}>{icon}</button>'
    )

    return srp_request_accept_icon


def get_srp_request_reject_icon(
    request: WSGIRequest,  # pylint: disable=unused-argument
    srp_link: SrpLink,
    srp_request: SrpRequest,
) -> str:
    """
    Get reject icon for an SRP request

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    button_request_reject_url = reverse(
        viewname="aasrp:ajax_srp_request_deny",
        args=[srp_link.srp_code, srp_request.request_code],
    )

    button_request_reject_state = (
        ' disabled="disabled"'
        if srp_request.request_status == SrpRequest.Status.REJECTED
        else ""
    )

    icon = '<i class="fa-solid fa-ban"></i>'
    title = _("Reject SRP request")

    srp_request_reject_icon = (
        f'<button data-link="{button_request_reject_url}" '
        'data-bs-tooltip="aa-srp" '
        'data-bs-toggle="modal" '
        'data-bs-target="#srp-request-reject" '
        'class="btn btn-warning btn-sm btn-icon-aasrp" '
        f'title="{title}"{button_request_reject_state}>{icon}</button>'
    )

    return srp_request_reject_icon


def get_srp_request_delete_icon(
    request: WSGIRequest,  # pylint: disable=unused-argument
    srp_link: SrpLink,
    srp_request: SrpRequest,
) -> str:
    """
    Get delete icon for an SRP request

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    button_request_delete_url = reverse(
        viewname="aasrp:ajax_srp_request_remove",
        args=[srp_link.srp_code, srp_request.request_code],
    )

    icon = '<i class="fa-solid fa-trash-can"></i>'
    title = _("Delete SRP request")

    srp_request_delete_icon = (
        f'<button data-link="{button_request_delete_url}" '
        'data-bs-tooltip="aa-srp" '
        'data-bs-toggle="modal" '
        'data-bs-target="#srp-request-remove" '
        'class="btn btn-danger btn-sm btn-icon-aasrp" '
        f'title="{title}">{icon}</button>'
    )

    return srp_request_delete_icon


@permissions_required(("aasrp.manage_srp", "aasrp.manage_srp_requests"))
def get_srp_request_action_icons(
    request: WSGIRequest, srp_link: SrpLink, srp_request: SrpRequest
) -> str:
    """
    Get action icons for srp requests

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    srp_request_action_icons = get_srp_request_details_icon(
        request=request, srp_link=srp_link, srp_request=srp_request
    )

    if srp_link.srp_status in (SrpLink.Status.ACTIVE, SrpLink.Status.CLOSED):
        srp_request_action_icons += "<br>"

        for icon_func in [get_srp_request_accept_icon, get_srp_request_reject_icon]:
            srp_request_action_icons += icon_func(
                request=request, srp_link=srp_link, srp_request=srp_request
            )

        if request.user.has_perm("aasrp.manage_srp"):
            srp_request_action_icons += get_srp_request_delete_icon(
                request=request, srp_link=srp_link, srp_request=srp_request
            )

    return srp_request_action_icons


def copy_to_clipboard_icon(data: str, title: str | Promise) -> SafeString:
    """
    Get copy to clipboard icon

    :param data: The data to be copied to the clipboard
    :type data: str
    :param title: The title for the icon
    :type title: str
    :return: The HTML for the copy to clipboard icon
    :rtype: SafeString
    """

    return render_to_string(
        template_name="aasrp/partials/common/copy-to-clipboard-icon.html",
        context={"data": data, "title": title},
    )



"""
Tests for form.py
"""

# Standard Library
from unittest.mock import patch

# Django
from django.core.exceptions import ValidationError
from django.test import TestCase

# AA SRP
from aasrp.constants import KILLBOARD_DATA
from aasrp.form import SrpRequestForm


class TestSrpRequestForm(TestCase):
    """
    Test SrpRequestForm
    """

    def test_validates_killboard_link_with_trailing_slash(self):
        """
        Test validates killboard link with trailing slash

        :return:
        :rtype:
        """

        form = SrpRequestForm(
            data={
                "killboard_link": "https://zkillboard.com/kill/123456/",
                "additional_info": "Details",
            }
        )
        form.cleaned_data = {"killboard_link": "https://zkillboard.com/kill/123456/"}

        with patch(
            "aasrp.form.KILLBOARD_DATA",
            {"zKillboard": KILLBOARD_DATA.get("zKillboard")},
        ):
            self.assertEqual(
                form.clean_killboard_link(), "https://zkillboard.com/kill/123456/"
            )

    def test_validates_killboard_link_without_trailing_slash(self):
        """
        Test validates killboard link without trailing slash and adds it if required

        :return:
        :rtype:
        """

        form = SrpRequestForm(
            data={
                "killboard_link": "https://zkillboard.com/kill/123456",
                "additional_info": "Details",
            }
        )
        form.cleaned_data = {"killboard_link": "https://zkillboard.com/kill/123456"}

        with patch(
            "aasrp.form.KILLBOARD_DATA",
            {"zKillboard": KILLBOARD_DATA.get("zKillboard")},
        ):
            self.assertEqual(
                form.clean_killboard_link(), "https://zkillboard.com/kill/123456/"
            )

    def test_raises_error_for_invalid_killboard_link(self):
        """
        Test raises error for invalid killboard link

        :return:
        :rtype:
        """

        form = SrpRequestForm(
            data={
                "killboard_link": "https://invalid.com/kill/123456/",
                "additional_info": "Details",
            }
        )
        form.cleaned_data = {"killboard_link": "https://invalid.com/kill/123456/"}

        with patch(
            "aasrp.form.KILLBOARD_DATA",
            {"zKillboard": KILLBOARD_DATA.get("zKillboard")},
        ):
            with self.assertRaises(ValidationError) as cm:
                form.clean_killboard_link()

            self.assertIn("Invalid link", str(cm.exception))

    def test_raises_error_for_non_killmail_link(self):
        """
        Test raises error for non-killmail link

        :return:
        :rtype:
        """

        form = SrpRequestForm(
            data={
                "killboard_link": "https://zkillboard.com/ship/123456/",
                "additional_info": "Details",
            }
        )
        form.cleaned_data = {"killboard_link": "https://zkillboard.com/ship/123456/"}

        with patch(
            "aasrp.form.KILLBOARD_DATA",
            {"zKillboard": KILLBOARD_DATA.get("zKillboard")},
        ):
            with self.assertRaises(ValidationError) as cm:
                form.clean_killboard_link()

            self.assertIn(
                "Invalid link. Please post a link to a kill mail.", str(cm.exception)
            )

    def test_raises_error_if_srp_request_already_exists(self):
        """
        Test raises error if SRP request already exists for the given killmail

        :return:
        :rtype:
        """

        form = SrpRequestForm(
            data={
                "killboard_link": "https://zkillboard.com/kill/123456/",
                "additional_info": "Details",
            }
        )
        form.cleaned_data = {"killboard_link": "https://zkillboard.com/kill/123456/"}

        with patch("aasrp.form.SrpRequest.objects.filter") as mock_filter:
            mock_filter.return_value.exists.return_value = True

            with self.assertRaises(ValidationError) as cm:
                form.clean_killboard_link()

            self.assertIn(
                "There is already an SRP request for this kill mail.", str(cm.exception)
            )



"""
Form definitions
"""

# Standard Library
import re

# Django
from django import forms
from django.forms import ModelForm
from django.utils.functional import Promise
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA SRP
from aasrp import __title__
from aasrp.constants import KILLBOARD_DATA
from aasrp.models import (
    FleetType,
    RequestComment,
    Setting,
    SrpLink,
    SrpRequest,
    UserSetting,
)

# Killboard URLs
zkillboard_base_url: str = KILLBOARD_DATA["zKillboard"]["base_url"]
evetools_base_url: str = KILLBOARD_DATA["EveTools"]["base_url"]
eve_kill_base_url: str = KILLBOARD_DATA["EVE-KILL"]["base_url"]

# Killboard regex
zkillboard_base_url_regex: str = KILLBOARD_DATA["zKillboard"]["base_url_regex"]
zkillboard_killmail_url_regex: str = KILLBOARD_DATA["zKillboard"]["killmail_url_regex"]
evetools_base_url_regex: str = KILLBOARD_DATA["EveTools"]["base_url_regex"]
evetools_killmail_url_regex: str = KILLBOARD_DATA["EveTools"]["killmail_url_regex"]
eve_kill_base_url_regex: str = KILLBOARD_DATA["EVE-KILL"]["base_url_regex"]
eve_kill_killmail_url_regex: str = KILLBOARD_DATA["EVE-KILL"]["killmail_url_regex"]

logger = LoggerAddTag(my_logger=get_extension_logger(__name__), prefix=__title__)


def get_mandatory_form_label_text(text: str | Promise) -> str:
    """
    Label text for mandatory form fields

    :param text:
    :type text:
    :return:
    :rtype:
    """

    required_text = _("This field is mandatory")
    required_marker = (
        f'<span aria-label="{required_text}" class="form-required-marker">*</span>'
    )

    return mark_safe(
        f'<span class="form-field-required">{text} {required_marker}</span>'
    )


class SrpLinkForm(ModelForm):
    """
    New SRP lnk form
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta definitions
        """

        model = SrpLink

        fields = ["srp_name", "fleet_time", "fleet_type", "fleet_doctrine", "aar_link"]
        labels = {
            "srp_name": get_mandatory_form_label_text(text=_("Fleet name")),
            "fleet_time": get_mandatory_form_label_text(text=_("Fleet time")),
            "fleet_type": _("Fleet type (optional)"),
            "fleet_doctrine": get_mandatory_form_label_text(text=_("Doctrine")),
            "aar_link": _("After action report link"),
        }
        querysets = {
            "fleet_type": FleetType.objects.filter(is_enabled=True),
        }
        widgets = {
            "srp_name": forms.TextInput(attrs={"placeholder": _("Fleet name")}),
            "fleet_time": forms.DateTimeInput(
                attrs={"autocomplete": "off", "placeholder": _("Fleet time")}
            ),
            "fleet_type": forms.Select(attrs={"placeholder": _("Fleet type")}),
            "fleet_doctrine": forms.TextInput(attrs={"placeholder": _("Doctrine")}),
            "aar_link": forms.TextInput(
                attrs={"placeholder": _("After action report link")}
            ),
        }


class SrpLinkUpdateForm(ModelForm):
    """
    Edit SRP link update form
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta definitions
        """

        model = SrpLink

        fields = ["aar_link"]
        labels = {"aar_link": _("After action report link")}


class SrpRequestForm(ModelForm):
    """
    SRP request form
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta definitions
        """

        model = SrpRequest

        fields = ["killboard_link", "additional_info"]
        help_texts = {
            "killboard_link": _(
                f"Find your kill mail on {zkillboard_base_url}, {evetools_base_url} or {eve_kill_base_url} and paste the link here."  # pylint: disable=line-too-long
            ),
            "additional_info": _(
                "Please tell us about the circumstances of your untimely demise. "
                "Who was the FC, what doctrine was called, have changes to the fit "
                "been requested and so on. Be as detailed as you can."
            ),
        }
        labels = {
            "killboard_link": get_mandatory_form_label_text(text=_("Killboard link")),
            "additional_info": get_mandatory_form_label_text(
                text=_("Additional information")
            ),
        }
        widgets = {
            "killboard_link": forms.URLInput(
                attrs={"placeholder": _("Killboard link")}
            ),
            "additional_info": forms.Textarea(
                attrs={
                    "placeholder": _("Additional information"),
                    "rows": 10,
                    "cols": 20,
                }
            ),
        }

    def clean_killboard_link(self):
        """
        Check if it's a link from one of the accepted kill boards and clean it

        :return:
        :rtype:
        """

        killboard_link = self.cleaned_data["killboard_link"]

        # Ensure the link ends with a trailing slash if required by the kill board
        for board, data in KILLBOARD_DATA.items():
            if (
                data["requires_trailing_slash"]
                and re.match(data["base_url_regex"], killboard_link)
                and not killboard_link.endswith("/")
            ):
                logger.debug(
                    f"Adding trailing slash to killboard link for {board}: {killboard_link}"
                )
                killboard_link += "/"
                self.cleaned_data["killboard_link"] = killboard_link

        logger.debug(f"Validating killboard link: {killboard_link}")

        # Define regex patterns for accepted kill boards and killmails
        killboard_patterns = [
            zkillboard_base_url_regex,
            evetools_base_url_regex,
            eve_kill_base_url_regex,
        ]
        killmail_patterns = [
            zkillboard_killmail_url_regex,
            evetools_killmail_url_regex,
            eve_kill_killmail_url_regex,
        ]

        # Check if it's a link from one of the accepted kill boards
        if not any(re.match(pattern, killboard_link) for pattern in killboard_patterns):
            logger.debug(
                f"Killboard link does not match any accepted kill board patterns: {killboard_link}"
            )
            raise forms.ValidationError(
                message=_(
                    f"Invalid link. Please use {zkillboard_base_url}, {evetools_base_url} or {eve_kill_base_url}"
                )
            )

        # Check if it's an actual killmail
        if not any(re.match(pattern, killboard_link) for pattern in killmail_patterns):
            logger.debug(
                f"Killboard link does not match any accepted killmail patterns: {killboard_link}"
            )
            raise forms.ValidationError(
                message=_("Invalid link. Please post a link to a kill mail.")
            )

        # Check if there is already an SRP request for this kill mail
        killmail_id = SrpRequest.objects.get_kill_id(killboard_link=killboard_link)

        logger.debug(f"Extracted killmail ID: {killmail_id}")

        if SrpRequest.objects.filter(
            killboard_link__icontains=f"/kill/{killmail_id}"
        ).exists():
            logger.debug(
                f"SRP request already exists for killmail ID {killmail_id} and link {killboard_link}"
            )
            raise forms.ValidationError(
                message=_(
                    "There is already an SRP request for this kill mail. "
                    "Please check if you got the right one."
                )
            )

        logger.debug(f"Killboard link validated: {killboard_link}")

        return killboard_link


class SrpRequestPayoutForm(forms.Form):
    """
    Change payout value
    """

    value = forms.CharField(label=_("SRP payout value"), max_length=254, required=True)


class SrpRequestRejectForm(ModelForm):
    """
    SRP request reject form
    """

    class Meta:
        """
        Meta definitions
        """

        model = RequestComment

        fields = ["comment"]
        help_texts = {
            "comment": _("Please provide the reason this SRP request is rejected."),
        }
        labels = {
            "comment": get_mandatory_form_label_text(text=_("Reject reason")),
        }
        widgets = {
            "comment": forms.Textarea(
                attrs={
                    "rows": 10,
                    "cols": 20,
                    # "placeholder": _("Reject reason"),
                    "required": "required",
                }
            ),
        }


class SrpRequestAcceptForm(ModelForm):
    """
    SRP request accept form
    """

    class Meta:
        """
        Meta definitions
        """

        model = RequestComment

        fields = ["comment"]
        help_texts = {
            "comment": _("Leave a comment for the requestor"),
        }
        labels = {
            "comment": _("Comment (optional)"),
        }
        widgets = {
            "comment": forms.Textarea(
                attrs={
                    "rows": 10,
                    "cols": 20,
                    # "placeholder": _("Reject reason"),
                }
            ),
        }


class SrpRequestAcceptRejectedForm(ModelForm):
    """
    SRP request accept rejected form
    """

    class Meta:
        """
        Meta definitions
        """

        model = RequestComment

        fields = ["comment"]
        help_texts = {
            "comment": _(
                "Please provide the reason why this former "
                "rejected SRP request is now accepted."
            ),
        }
        labels = {
            "comment": get_mandatory_form_label_text(text=_("Comment")),
        }
        widgets = {
            "comment": forms.Textarea(
                attrs={
                    "rows": 10,
                    "cols": 20,
                    # "placeholder": _("Reject reason"),
                    "required": "required",
                }
            ),
        }


class UserSettingsForm(ModelForm):
    """
    User settings form
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta definitions
        """

        model = UserSetting

        fields = ["disable_notifications"]
        labels = {
            "disable_notifications": _(
                "Disable notifications. "
                "(Auth and Discord, if a relevant module is installed)"
            ),
        }


class SettingAdminForm(forms.ModelForm):
    """
    Form definitions for the FleetType form in admin
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta
        """

        model = Setting

        fields = "__all__"
        widgets = {"default_embed_color": forms.TextInput(attrs={"type": "color"})}



"""
Constants
"""

# Standard Library
from enum import Enum

# Third Party
from requests.__version__ import __version__ as requests_version

# AA SRP
from aasrp import __app_name_useragent__, __github_url__
from aasrp import __version__ as app_version

# All internal URLs need to start with this prefix
INTERNAL_URL_PREFIX = "-"


class UserAgent(Enum):
    """
    UserAgent
    """

    REQUESTS = f"{__app_name_useragent__}/{app_version} (+{__github_url__}) requests/{requests_version}"


SRP_REQUEST_NOTIFICATION_INQUIRY_NOTE = (
    "If you have any questions regarding your SRP request, feel free to contact your "
    "request reviser. Please make sure to always add the SRP code and the "
    "request code with your inquiry."
)

# Killboard URLs and regex
KILLBOARD_DATA = {
    "zKillboard": {
        "base_url": "https://zkillboard.com/",
        "api_url": "https://zkillboard.com/api/",
        "base_url_regex": r"^http[s]?:\/\/zkillboard\.com\/",
        "killmail_url_regex": r"^http[s]?:\/\/zkillboard\.com\/kill\/\d+\/",
        "requires_trailing_slash": True,
    },
    "EveTools": {
        "base_url": "https://kb.evetools.org/",
        "base_url_regex": r"^http[s]?:\/\/kb\.evetools\.org\/",
        "killmail_url_regex": r"^http[s]?:\/\/kb\.evetools\.org\/kill\/\d+",
        "requires_trailing_slash": True,
    },
    "EVE-KILL": {
        "base_url": "https://eve-kill.com/",
        "base_url_regex": r"^http[s]?:\/\/eve-kill\.com\/",
        "killmail_url_regex": r"^http[s]?:\/\/eve-kill\.com\/kill\/\d+",
        "requires_trailing_slash": True,
    },
}


class DiscordEmbedColor(Enum):
    """
    Discord embed colors
    """

    INFO = 0x5BC0DE
    SUCCESS = 0x5CB85C
    WARNING = 0xF0AD4E
    DANGER = 0xD9534F


DISCORD_EMBED_COLOR_MAP = {
    color.name.lower(): color.value for color in DiscordEmbedColor
}



"""
Test cases for the helper functions in the aasrp.helper.srp_data module.
"""

# Standard Library
from unittest.mock import patch

# Django
from django.test import TestCase

# AA SRP
from aasrp.helper.icons import copy_to_clipboard_icon
from aasrp.helper.srp_data import (
    payout_amount_html,
    request_code_html,
    request_fleet_details_html,
)
from aasrp.models import SrpLink, SrpRequest


class TestPayoutAmountHtml(TestCase):
    """
    Test cases for the payout_amount_html function.
    """

    @patch("aasrp.helper.srp_data._")
    def test_payout_amount_html_returns_expected_html(self, mock_translate):
        """
        Test returning expected HTML for payout amount.

        :param mock_translate:
        :type mock_translate:
        :return:
        :rtype:
        """

        mock_translate.return_value = "Copy payout amount to clipboard"

        payout_amount = 1000000
        expected_html = (
            '<span class="srp-payout d-flex justify-content-end align-items-baseline">'
            '<span class="srp-payout-tooltip"><span class="srp-payout-amount d-block cursor-pointer">'
            "#payout_amount_localized#</span></span><sup>"
            '<span class="copy-to-clipboard-icon">'
            '<i class="copy-to-clipboard fa-regular fa-copy ms-2 cursor-pointer" '
            'data-clipboard-text="1000000" data-bs-tooltip="aa-srp" aria-label="Copy payout amount to clipboard" '
            'title="Copy payout amount to clipboard"></i></span></sup></span>'
        )

        result = payout_amount_html(payout_amount)

        self.assertHTMLEqual(result, expected_html)

    @patch("aasrp.helper.srp_data._")
    def test_payout_amount_html_handles_negative_payout(self, mock_translate):
        """
        Test returning expected HTML for negative payout amount.

        :param mock_translate:
        :type mock_translate:
        :return:
        :rtype:
        """

        mock_translate.return_value = "Copy payout amount to clipboard"

        payout_amount = -500
        expected_html = (
            '<span class="srp-payout d-flex justify-content-end align-items-baseline">'
            '<span class="srp-payout-tooltip"><span class="srp-payout-amount d-block cursor-pointer">'
            "#payout_amount_localized#</span></span><sup>"
            '<span class="copy-to-clipboard-icon">'
            '<i class="copy-to-clipboard fa-regular fa-copy ms-2 cursor-pointer" '
            'data-clipboard-text="-500" data-bs-tooltip="aa-srp" aria-label="Copy payout amount to clipboard" '
            'title="Copy payout amount to clipboard"></i></span></sup></span>'
        )

        result = payout_amount_html(payout_amount)

        self.assertHTMLEqual(result, expected_html)

    @patch("aasrp.helper.srp_data._")
    def test_payout_amount_html_handles_zero_payout(self, mock_translate):
        """
        Test returning expected HTML for zero payout amount.

        :param mock_translate:
        :type mock_translate:
        :return:
        :rtype:
        """

        mock_translate.return_value = "Copy payout amount to clipboard"

        payout_amount = 0
        expected_html = (
            '<span class="srp-payout d-flex justify-content-end align-items-baseline">'
            '<span class="srp-payout-tooltip"><span class="srp-payout-amount d-block cursor-pointer">'
            "#payout_amount_localized#</span></span><sup>"
            '<span class="copy-to-clipboard-icon">'
            '<i class="copy-to-clipboard fa-regular fa-copy ms-2 cursor-pointer" '
            'data-clipboard-text="0" data-bs-tooltip="aa-srp" aria-label="Copy payout amount to clipboard" '
            'title="Copy payout amount to clipboard"></i></span></sup></span>'
        )

        result = payout_amount_html(payout_amount)

        self.assertHTMLEqual(result, expected_html)


class TestRequestCodeHtml(TestCase):
    """
    Test cases for the request_code_html function.
    """

    def test_request_code_html_returns_correct_html(self):
        """
        Test returning correct HTML for request code.

        :return:
        :rtype:
        """

        result = request_code_html("ABC123")

        icon = copy_to_clipboard_icon(
            data="ABC123", title="Copy request code to clipboard"
        )

        self.assertEqual(result, f"ABC123<sup>{icon}</sup>")


class TestRequestFleetDetailsHtml(TestCase):
    """
    Test cases for the request_fleet_details_html function.
    """

    def test_request_fleet_details_html_returns_correct_html(self):
        """
        Test returning correct HTML for fleet details.

        :return:
        :rtype:
        """

        srp_link = SrpLink(srp_name="Fleet Alpha", srp_code="FA123")
        srp_request = SrpRequest(srp_link=srp_link, request_code="REQ123")
        result = request_fleet_details_html(srp_request)

        self.assertIn("<p>Fleet Alpha</p>", result)
        self.assertIn("SRP code: FA123", result)
        self.assertIn("Request code: REQ123", result)

    def test_request_fleet_details_html_handles_empty_srp_name(self):
        """
        Test returning correct HTML for fleet details with empty SRP name.

        :return:
        :rtype:
        """

        srp_link = SrpLink(srp_name="", srp_code="FA123")
        srp_request = SrpRequest(srp_link=srp_link, request_code="REQ123")
        result = request_fleet_details_html(srp_request)

        self.assertIn("<p></p>", result)
        self.assertIn("SRP code: FA123", result)
        self.assertIn("Request code: REQ123", result)

    def test_request_fleet_details_html_handles_empty_request_code(self):
        """
        Test returning correct HTML for fleet details with empty request code.

        :return:
        :rtype:
        """

        srp_link = SrpLink(srp_name="Fleet Alpha", srp_code="FA123")
        srp_request = SrpRequest(srp_link=srp_link, request_code="")
        result = request_fleet_details_html(srp_request)

        self.assertIn("<p>Fleet Alpha</p>", result)
        self.assertIn("SRP code: FA123", result)
        self.assertIn("Request code: ", result)

    def test_request_fleet_details_html_handles_special_characters(self):
        """
        Test returning correct HTML for fleet details with special characters.

        :return:
        :rtype:
        """

        srp_link = SrpLink(srp_name="Fleet @lpha", srp_code="FA!23")
        srp_request = SrpRequest(srp_link=srp_link, request_code="REQ!23")
        result = request_fleet_details_html(srp_request)

        self.assertIn("<p>Fleet @lpha</p>", result)
        self.assertIn("SRP code: FA!23", result)
        self.assertIn("Request code: REQ!23", result)



"""
App settings
"""

# Django
from django.apps import apps
from django.conf import settings

# Port used to communicate with Discord Proxy
DISCORDPROXY_PORT = getattr(settings, "DISCORDPROXY_PORT", 50051)

# Host used to communicate with Discord Proxy
DISCORDPROXY_HOST = getattr(settings, "DISCORDPROXY_HOST", "localhost")

# Timeout for Discord Proxy communication
DISCORDPROXY_TIMEOUT = getattr(settings, "DISCORDPROXY_TIMEOUT", 300)


def allianceauth_discordbot_installed() -> bool:
    """
    Check if allianceauth-discordbot is installed and active

    :return:
    :rtype:
    """

    return apps.is_installed(app_name="aadiscordbot")


def aa_discordnotify_installed() -> bool:
    """
    Check if allianceauth-discordbot is installed and active

    :return:
    :rtype:
    """

    return apps.is_installed(app_name="discordnotify")


def discordproxy_installed() -> bool:
    """
    Check if discordproxy is installed by trying to import the DiscordClient

    :return:
    :rtype:
    """

    try:
        # Third Party
        # pylint: disable=import-outside-toplevel, unused-import
        from discordproxy.client import DiscordClient  # noqa: F401
    except ModuleNotFoundError:
        return False

    return True



"""
Test cases for the helper.numbers module in the aasrp package.
"""

# Django
from django.test import TestCase, override_settings

# AA SRP
from aasrp.helper.numbers import l10n_number_format


class TestL10nNumberFormat(TestCase):
    """
    Test cases for the l10n_number_format function.
    """

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_integer_with_default_decimals_locale_en(self):
        """
        Test localization of an integer with default decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(1234), "1,234")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_integer_with_default_decimals_locale_de(self):
        """
        Test localization of an integer with default decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(1234), "1.234")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_float_with_default_decimals_locale_en(self):
        """
        Test localization of a float with default decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(1234.5678), "1,235")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_float_with_default_decimals_locale_de(self):
        """
        Test localization of a float with default decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(1234.5678), "1.235")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_negative_number_with_default_decimals_locale_en(self):
        """
        Test localization of a negative number with default decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(-1234.5678), "-1,235")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_negative_number_with_default_decimals_locale_de(self):
        """
        Test localization of a negative number with default decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(-1234.5678), "-1.235")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_zero_with_default_decimals_locale_en(self):
        """
        Test localization of zero with default decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(0), "0")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_zero_with_default_decimals_locale_de(self):
        """
        Test localization of zero with default decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(0), "0")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_large_number_with_default_decimals_locale_en(self):
        """
        Test localization of a large number with default decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(1234567890.123456), "1,234,567,890")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_large_number_with_default_decimals_locale_de(self):
        """
        Test localization of a large number with default decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(1234567890.123456), "1.234.567.890")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_small_number_with_default_decimals_locale_en(self):
        """
        Test localization of a small number with default decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(0.0001234), "0")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_small_number_with_default_decimals_locale_de(self):
        """
        Test localization of a small number with default decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(0.0001234), "0")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_none_value_with_default_decimals_locale_en(self):
        """
        Test localization of a None value with default decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(None), "")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_none_value_with_default_decimals_locale_de(self):
        """
        Test localization of a None value with default decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(None), "")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_integer_with_specified_decimals_locale_en(self):
        """
        Test localization of an integer with specified decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(1234, 2), "1,234.00")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_integer_with_specified_decimals_locale_de(self):
        """
        Test localization of an integer with specified decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(1234, 2), "1.234,00")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_float_with_specified_decimals_locale_en(self):
        """
        Test localization of a float with specified decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(1234.5678, 2), "1,234.57")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_float_with_specified_decimals_locale_de(self):
        """
        Test localization of a float with specified decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(1234.5678, 2), "1.234,57")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_negative_number_with_specified_decimals_locale_en(self):
        """
        Test localization of a negative number with specified decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(-1234.5678, 2), "-1,234.57")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_negative_number_with_specified_decimals_locale_de(self):
        """
        Test localization of a negative number with specified decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(-1234.5678, 2), "-1.234,57")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_zero_with_specified_decimals_locale_en(self):
        """
        Test localization of zero with specified decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(0, 2), "0.00")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_zero_with_specified_decimals_locale_de(self):
        """
        Test localization of zero with specified decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(0, 2), "0,00")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_large_number_with_specified_decimals_locale_en(self):
        """
        Test localization of a large number with specified decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(1234567890.123456, 2), "1,234,567,890.12")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_large_number_with_specified_decimals_locale_de(self):
        """
        Test localization of a large number with specified decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(1234567890.123456, 2), "1.234.567.890,12")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_small_number_with_specified_decimals_locale_en(self):
        """
        Test localization of a small number with specified decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(0.0001234, 6), "0.000123")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_small_number_with_specified_decimals_locale_de(self):
        """
        Test localization of a small number with specified decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(0.0001234, 6), "0,000123")

    @override_settings(LANGUAGE_CODE="en")
    def test_localizes_none_value_with_specified_decimals_locale_en(self):
        """
        Test localization of a None value with specified decimals in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(None, 2), "")

    @override_settings(LANGUAGE_CODE="de")
    def test_localizes_none_value_with_specified_decimals_locale_de(self):
        """
        Test localization of a None value with specified decimals in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(l10n_number_format(None, 2), "")
